/* 
 * Stellarium
 * Copyright (C) 2002 Fabien Chreau
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "draw.h"
#include "astroLib/MathOps.h"
#include "stellarium.h"

float DmeriParal[51][2];	// For grids drawing optimisation

// precalculation of the grid points
void InitMeriParal(void)
{	for(int j=0;j<51;j++)
	{	DmeriParal[j][1]=cos(j*2*PI/50)*20;
		DmeriParal[j][2]=sin(j*2*PI/50)*20;
	}
}

// Draw the cardinals points : N S E O and Z (Zentih) N (Nadir)
void DrawCardinaux(void)
{	glPushMatrix();
	glColor3f(0.8f, 0.1f, 0.1f);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBindTexture(GL_TEXTURE_2D,texIds[6]);		//Nord
	glTranslatef(0.0f,0.0f,30.0f);
	glBegin(GL_QUADS );
		glTexCoord2f(0.0f,0.0f);	glVertex3f( 1.0f,-1.0f,0.0f);	// Bas Gauche
		glTexCoord2f(1.0f,0.0f);	glVertex3f(-1.0f,-1.0f,0.0f);	// Bas Droite
		glTexCoord2f(1.0f,1.0f);	glVertex3f(-1.0f, 1.0f,0.0f);	// Haut Droit
		glTexCoord2f(0.0f,1.0f);	glVertex3f( 1.0f, 1.0f,0.0f);	// Haut Gauche
	glEnd ();

	glBindTexture(GL_TEXTURE_2D, texIds[7]);	//Sud
	glTranslatef(0.0f,0.0f,-60.0f);
	glBegin(GL_QUADS );
		glTexCoord2f(0.0f,0.0f);	glVertex3f(-1.0f,-1.0f,0.0f);	// Bas Gauche
		glTexCoord2f(1.0f,0.0f);	glVertex3f( 1.0f,-1.0f,0.0f);	// Bas Droite
		glTexCoord2f(1.0f,1.0f);	glVertex3f( 1.0f, 1.0f,0.0f);	// Haut Droit
		glTexCoord2f(0.0f,1.0f);	glVertex3f(-1.0f, 1.0f,0.0f);	// Haut Gauche
	glEnd ();

	glBindTexture(GL_TEXTURE_2D, texIds[9]);	//Ouest
	glTranslatef(30.0f,0.0f,30.0f);
	glBegin(GL_QUADS );
		glTexCoord2f(0.0f,0.0f);	glVertex3f(0.0f,-1.0f,-1.0f);	// Bas Gauche
		glTexCoord2f(1.0f,0.0f);	glVertex3f(0.0f,-1.0f, 1.0f);	// Bas Droite
		glTexCoord2f(1.0f,1.0f);	glVertex3f(0.0f, 1.0f, 1.0f);	// Haut Droit
		glTexCoord2f(0.0f,1.0f);	glVertex3f(0.0f, 1.0f,-1.0f);	// Haut Gauche
	glEnd ();

	glBindTexture(GL_TEXTURE_2D, texIds[8]);	//Est
	glTranslatef(-60.0f,0.0f,0.0f);
	glBegin(GL_QUADS );
		glTexCoord2f(0.0f,0.0f);	glVertex3f(0.0f,-1.0f, 1.0f);	// Bas Gauche
		glTexCoord2f(1.0f,0.0f);	glVertex3f(0.0f,-1.0f,-1.0f);	// Bas Droite
		glTexCoord2f(1.0f,1.0f);	glVertex3f(0.0f, 1.0f,-1.0f);	// Haut Droit
		glTexCoord2f(0.0f,1.0f);	glVertex3f(0.0f, 1.0f, 1.0f);	// Haut Gauche
	glEnd ();

	glBindTexture(GL_TEXTURE_2D, texIds[10]);	//Zenith
	glTranslatef(30.0f,30.0f,0.0f);
	glBegin(GL_QUADS );
		glTexCoord2f(0.0f,0.0f);	glVertex3f( 1.0f, 0.0f,-1.0f);	// Bas Gauche
		glTexCoord2f(1.0f,0.0f);	glVertex3f( 1.0f, 0.0f, 1.0f);	// Bas Droite
		glTexCoord2f(1.0f,1.0f);	glVertex3f(-1.0f, 0.0f, 1.0f);	// Haut Droit
		glTexCoord2f(0.0f,1.0f);	glVertex3f(-1.0f, 0.0f,-1.0f);	// Haut Gauche
	glEnd ();

	glBindTexture(GL_TEXTURE_2D, texIds[11]);	//Nadir
	glTranslatef(0.0f,-60.0f,0.0f);
	glBegin(GL_QUADS );
		glTexCoord2f(0.0f,0.0f);	glVertex3f( 1.0f, 0.0f,-1.0f);	// Bas Gauche
		glTexCoord2f(1.0f,0.0f);	glVertex3f( 1.0f, 0.0f, 1.0f);	// Bas Droite
		glTexCoord2f(1.0f,1.0f);	glVertex3f(-1.0f, 0.0f, 1.0f);	// Haut Droit
		glTexCoord2f(0.0f,1.0f);	glVertex3f(-1.0f, 0.0f,-1.0f);	// Haut Gauche
	glEnd ();
	glPopMatrix();
}

// Draw the milky way : used too to 'clear' the buffer
void DrawMilkyWay(void)
{	if (!global.FlagMilkyWay)
	{	glClear(GL_COLOR_BUFFER_BIT);
		return;
	}
	glPushMatrix();
	glColor3f(0.12f-global.SkyBrightness, 0.12f-global.SkyBrightness, 0.18f-global.SkyBrightness);
	glEnable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
	glBindTexture(GL_TEXTURE_2D, texIds[2]);
	glRotatef(206,1,0,0);
	glRotatef(-16,0,1,0);
	glRotatef(-84,0,0,1);
	GLUquadricObj * MilkyWay=gluNewQuadric();
	gluQuadricTexture(MilkyWay,GL_TRUE);
	gluSphere(MilkyWay,1000,15,15);
	gluDeleteQuadric(MilkyWay);
	glPopMatrix();
}

// Draw the equatorial merididians
void DrawMeridiens(void)
{	glPushMatrix();	
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
	//glEnable (GL_LINE_STIPPLE);
	//glLineStipple (1, 0x0101);  /*  dotted  */
	//glLineStipple (1, 0x00FF);  /*  dashed  */
	glLineStipple (1, 0x1C47);  /*  dash/dot/dash  */
	float coef;
	coef=rand();
	coef/=RAND_MAX;
	glColor3f(0.12*coef+0.18,0.015*coef+0.2,0.015*coef+0.2);
	for(int i=0;i<12;i++)
	{	glRotatef(360/24,0,1,0);
		for(int j=0;j<50;j++)
		{	glBegin (GL_LINES);
				glVertex3f(DmeriParal[j][1],DmeriParal[j][2], 0.0f);
				glVertex3f(DmeriParal[j+1][1],DmeriParal[j+1][2], 0.0f);
			glEnd ();
		}
	}
	//glDisable (GL_LINE_STIPPLE);
	glPopMatrix();
}

// Draw the azimuthal merididians
void DrawMeridiensAzimut(void)
{	glPushMatrix();
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
	glEnable (GL_LINE_STIPPLE);
	//glLineStipple (1, 0x0101);  /*  dotted  */
	glLineStipple (1, 0x00FF);  /*  dashed  */
	//glLineStipple (1, 0x1C47);  /*  dash/dot/dash  */
	float coef;
	coef=rand();
	coef/=RAND_MAX;
	glColor3f(0.015*coef+0.2,0.12*coef+0.18,0.015*coef+0.2);
	for(int i=0;i<12;i++)
	{	glRotatef(360/24,0,1,0);
		for(int j=0;j<50;j++)
		{	glBegin (GL_LINES);
				glVertex3f(DmeriParal[j][1],DmeriParal[j][2], 0.0f);
				glVertex3f(DmeriParal[j+1][1],DmeriParal[j+1][2], 0.0f);
			glEnd ();
		}
	}
	glDisable (GL_LINE_STIPPLE);
	glPopMatrix();
}

// Draw the equatorial parallels
void DrawParallels(void)
{	glPushMatrix();
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
	//glEnable (GL_LINE_STIPPLE);
	//glLineStipple (1, 0x0101);  /*  dotted  */
	//glLineStipple (1, 0x00FF);  /*  dashed  */
	glLineStipple (1, 0x1C47);  /*  dash/dot/dash  */
	float coef;
	coef=rand();
	coef/=RAND_MAX;
	glColor3f(0.2*coef+0.2,0.025*coef+0.2,0.025*coef+0.2);
	for(int i=0;i<18;i++)
	{	for(int j=0;j<50;j++)
		{	float r=cos((i-9)*PI/18);
			float h=sin((i-9)*PI/18)*20;
			glBegin (GL_LINES);
				glVertex3f(DmeriParal[j][1]*r,h,DmeriParal[j][2]*r);
				glVertex3f(DmeriParal[j+1][1]*r,h,DmeriParal[j+1][2]*r);
			glEnd ();
		}
	}
	//glDisable (GL_LINE_STIPPLE);
	glPopMatrix();
}

// Draw the azimuthal parallels
void DrawParallelsAzimut(void)
{	glPushMatrix();
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
	glEnable (GL_LINE_STIPPLE);
	//glLineStipple (1, 0x0101);  /*  dotted  */
	glLineStipple (1, 0x00FF);  /*  dashed  */
	//glLineStipple (1, 0x1C47);  /*  dash/dot/dash  */
	float coef;
	coef=rand();
	coef/=RAND_MAX;
	glColor3f(0.015*coef+0.2,0.12*coef+0.18,0.015*coef+0.2);
	for(int i=0;i<18;i++)
	{	for(int j=0;j<50;j++)
		{	float r=cos((i-9)*PI/18);
			float h=sin((i-9)*PI/18)*20;
			glBegin (GL_LINES);
				glVertex3f(DmeriParal[j][1]*r,h,DmeriParal[j][2]*r);
				glVertex3f(DmeriParal[j+1][1]*r,h,DmeriParal[j+1][2]*r);
			glEnd ();
		}
	}
	glDisable (GL_LINE_STIPPLE);
	glPopMatrix();
}

// Draw the celestrial equator
void DrawEquator(void)
{	glPushMatrix();
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
	float coef;
	coef=rand();
	coef/=RAND_MAX;
	glColor3f(0.2*coef+0.3,0.025*coef+0.1,0.025*coef+0.1);
	for(int j=0;j<50;j++)
	{	glBegin (GL_LINES);
			glVertex3f(DmeriParal[j][1]*20,0,DmeriParal[j][2]*20);
			glVertex3f(DmeriParal[j+1][1]*20,0,DmeriParal[j+1][2]*20);
		glEnd ();
	}
	glPopMatrix();
}

// Draw the ecliptic line
void DrawEcliptic(void)
{	glPushMatrix();
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
	float coef;
	coef=rand();
	coef/=RAND_MAX;
	glColor3f(0.025*coef+0.2,0.025*coef+0.2,0.025*coef+0.0);
	
	const double centuries = Astro::toMillenia(global.JDay);
	const double obliquity = AstroOps::meanObliquity(centuries);
	glRotatef(obliquity*180/PI,0,0,1);
	for(int j=0;j<50;j++)
	{	glBegin (GL_LINES);
			glVertex3f(DmeriParal[j][1]*20,0,DmeriParal[j][2]*20);
			glVertex3f(DmeriParal[j+1][1]*20,0,DmeriParal[j+1][2]*20);
		glEnd ();
	}
	glPopMatrix();
}

// Draw the horizon fog
void DrawFog(void)
{	glPushMatrix();
	glColor4f(0.1f+0.2f*global.SkyBrightness, 0.15f+0.2f*global.SkyBrightness, 0.15f+0.2f*global.SkyBrightness,0.0);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBindTexture(GL_TEXTURE_2D, texIds[3]);
	glTranslatef(0,-20,0);
	glRotatef(-90,1,0,0);
	GLUquadricObj * Horizon=gluNewQuadric();
	gluQuadricTexture(Horizon,GL_TRUE);
	gluCylinder(Horizon,800,800,300,10,1);
	gluDeleteQuadric(Horizon);
	glPopMatrix();	
}

// Draw a point... (used for tests)
void DrawPoint(float X,float Y,float Z)
{		glColor3f(0.5, 1.0, 0.5);
		glDisable(GL_TEXTURE_2D);
		glEnable(GL_BLEND);
		glPointSize(5);
		glBegin(GL_POINTS);
			glVertex3f(X,Y,Z);
		glEnd();
}

// Draw the pointer
void DrawPointer(vec3_t obj, float size, vec3_t RGB, int ObjType)
{	double x,y;
	Project(obj[0],obj[1],obj[2],x,y);
	setOrthographicProjection(global.X_Resolution, global.Y_Resolution);
	glPushMatrix();
	glLoadIdentity();
	glColor3fv(RGB);
	if (ObjType==0 )	// star
	{	glBindTexture (GL_TEXTURE_2D, texIds[12]);
		glEnable(GL_TEXTURE_2D);
		glEnable(GL_BLEND);
		glTranslatef(x, global.Y_Resolution-y,0.0f);
		glRotatef((float)global.Timefr/20,0,0,1);
		glBegin(GL_QUADS );
			glTexCoord2f(0.0f,0.0f);	glVertex3f(-13,-13,0);		//Bas Gauche
			glTexCoord2f(1.0f,0.0f);	glVertex3f(13,-13,0);		//Bas Droite
			glTexCoord2f(1.0f,1.0f);	glVertex3f(13,13,0);		//Haut Droit
			glTexCoord2f(0.0f,1.0f);	glVertex3f(-13,13,0);		//Haut Gauche
		glEnd ();
	}
	if (ObjType==1 || ObjType==2)	// nebula : 1 or planet : 2
	{	if (ObjType==2)
		{	glBindTexture(GL_TEXTURE_2D, texIds[26]);
			size+=global.Fov/4;
			size+=size*sin((float)global.Timefr/400)/200*global.Fov;
		}
		if (ObjType==1)
		{	glBindTexture(GL_TEXTURE_2D, texIds[27]);
			size+=size*sin((float)global.Timefr/400)*global.Fov/200;
		}

		glEnable(GL_TEXTURE_2D);
		glEnable(GL_BLEND);
		glTranslatef(x, global.Y_Resolution-y,0.0f);
		if (ObjType==2) glRotatef((float)global.Timefr/100,0,0,-1);

		glTranslatef(-size/2, -size/2,0.0f);
		glRotatef(90,0,0,1);
		glBegin(GL_QUADS );
			glTexCoord2f(0.0f,0.0f);	glVertex3f(-10,-10,0);		//Bas Gauche
			glTexCoord2f(1.0f,0.0f);	glVertex3f(10,-10,0);		//Bas Droite
			glTexCoord2f(1.0f,1.0f);	glVertex3f(10,10,0);		//Haut Droit
			glTexCoord2f(0.0f,1.0f);	glVertex3f(-10,10,0);		//Haut Gauche
		glEnd ();

		glRotatef(-90,0,0,1);
		glTranslatef(0,size,0.0f);
		glBegin(GL_QUADS );
			glTexCoord2f(0.0f,0.0f);	glVertex3f(-10,-10,0);		//Bas Gauche
			glTexCoord2f(1.0f,0.0f);	glVertex3f(10,-10,0);		//Bas Droite
			glTexCoord2f(1.0f,1.0f);	glVertex3f(10,10,0);		//Haut Droit
			glTexCoord2f(0.0f,1.0f);	glVertex3f(-10,10,0);		//Haut Gauche
		glEnd ();

		glRotatef(-90,0,0,1);
		glTranslatef(0, size,0.0f);
		glBegin(GL_QUADS );
			glTexCoord2f(0.0f,0.0f);	glVertex3f(-10,-10,0);		//Bas Gauche
			glTexCoord2f(1.0f,0.0f);	glVertex3f(10,-10,0);		//Bas Droite
			glTexCoord2f(1.0f,1.0f);	glVertex3f(10,10,0);		//Haut Droit
			glTexCoord2f(0.0f,1.0f);	glVertex3f(-10,10,0);		//Haut Gauche
		glEnd ();

		glRotatef(-90,0,0,1);
		glTranslatef(0,size,0);
		glBegin(GL_QUADS );
			glTexCoord2f(0.0f,0.0f);	glVertex3f(-10,-10,0);		//Bas Gauche
			glTexCoord2f(1.0f,0.0f);	glVertex3f(10,-10,0);		//Bas Droite
			glTexCoord2f(1.0f,1.0f);	glVertex3f(10,10,0);		//Haut Droit
			glTexCoord2f(0.0f,1.0f);	glVertex3f(-10,10,0);		//Haut Gauche
		glEnd ();
	}
		glPopMatrix();
		resetPerspectiveProjection();
}

// Draw the mountains with a few pieces of a big texture
void DrawDecor(int nb)
{	glPushMatrix();
	glColor3f(0.2f+0.5f*global.SkyBrightness, 0.2f+0.8f*global.SkyBrightness, 0.2f+0.5f*global.SkyBrightness);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);

	float delta=1.98915/nb; // environ egal  sin((float)(PI/8))/cos((float)(PI/8))*10/2;
	for (int ntex=0;ntex<4*nb;ntex++)
	{	glBindTexture(GL_TEXTURE_2D, texIds[31+ntex%4]);
		for (int nDecoupe=0;nDecoupe<4;nDecoupe++)
		{	glBegin(GL_QUADS );
				//Haut Gauche
				glTexCoord2f((float)nDecoupe/4,1.0f);		
				glVertex3f(10.0f,1.5f,-delta);
				//Bas Gauche
				glTexCoord2f((float)nDecoupe/4,0.0f);		
				glVertex3f(10.0f,-1.0f,-delta);
				//Bas Droit
				glTexCoord2f((float)nDecoupe/4+0.25,0.0f);
				glVertex3f(10.0f,-1.0f,delta);
				//Haut Droit
				glTexCoord2f((float)nDecoupe/4+0.25,1.0f);
				glVertex3f(10.0f,1.5f,delta);
			glEnd ();
			glRotatef(22.5/nb,0,-1,0);
		}
	}
	glPopMatrix();
}

// Draw the sky
void DrawAtmosphere(void)
{	glPushMatrix();	
	glColor4f(0.6*global.SkyBrightness, 0.6*global.SkyBrightness, 1.0*global.SkyBrightness,2*global.SkyBrightness);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBindTexture(GL_TEXTURE_2D, texIds[4]);

	glRotatef(-90,1,0,0);
	GLUquadricObj * Atmosphere=gluNewQuadric();
	gluQuadricTexture(Atmosphere,GL_TRUE);
	gluSphere(Atmosphere,10,20,20);
	gluDeleteQuadric(Atmosphere);
	glPopMatrix();
}

// Draw the ground
void DrawGround(void)
{	glPushMatrix();
	glColor3f(0.12+0.6f*global.SkyBrightness, 0.12+0.6f*global.SkyBrightness, 0.2+0.7f*global.SkyBrightness);
	glEnable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
	glBindTexture(GL_TEXTURE_2D, texIds[1]);
	glBegin (GL_QUADS);
        glTexCoord2f (0.0f,0.0f);
        glVertex3f (-1000.0f, -5.0f, -1000.0f);
        glTexCoord2f (1.0f, 0.0f);
		glVertex3f(-1000.0f, -5.0f,  1000.0f);
        glTexCoord2f (1.0f, 1.0f);
		glVertex3f( 1000.0f, -5.0f,  1000.0f);
        glTexCoord2f (0.0f, 1.0f);
		glVertex3f( 1000.0f, -5.0f, -1000.0f);
    glEnd ();
	glPopMatrix();
}

/*
// Print info screen
void DrawInfo(void)
{	glColor3f(0.8f,0.3f,0.3f);
	float yt=winH/2-180*(float)global.Y_Resolution/1024;
	centerPrint(yt,"APP_NAME");
	yt+=lineSizeOf(FONT1)*3;
	centerPrint(yt,"Copyright (c) 2002 Fabien Chereau");
	yt+=lineSizeOf(FONT1)*2;
	centerPrint(yt,"January 2002");
	yt+=lineSizeOf(FONT1)*2;
	centerPrint(yt,"Please send bug report & comments to stellarium@free.fr");
	yt+=lineSizeOf(FONT1)*3;
	centerPrint(yt,"This software is free but if you have too much");
	yt+=lineSizeOf(FONT1);
	centerPrint(yt,"money, just send it to :");
	yt+=lineSizeOf(FONT1)*2;
	centerPrint(yt,"Club Astronomie de l'INSA de Lyon");
	yt+=lineSizeOf(FONT1);
	centerPrint(yt,"20, Av. Albert Einstein");
	yt+=lineSizeOf(FONT1);
	centerPrint(yt,"69621 Villeurbanne, FRANCE");
}
*/
