/* 
 * Stellarium
 * Copyright (C) 2002 Fabien Chreau
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "nebula.h"
#include "texture.h"
#include "stellarium.h"

double precalculatedFOVNeb;

Nebula::Nebula()
{	incLum = rand()%100;
}

Nebula::~Nebula()
{	delete matTransfo;
}

int Nebula::Read(FILE * catalogue)
// Lis les infos de la nbuleuse dans le fichier et calcule x,y et z;
{	int rahr;
	float ramin;
	int dedeg;
	int demin;
	matTransfo=new float[16];	// Used to store the precalc transfos matrix used to draw the star

	if (fscanf(catalogue,"%u %u %s %s %d %f %d %d %f %f %f %s\n",&NGC,&Messier, Type, Constellation, &rahr, &ramin,&dedeg,&demin,&Mag,&Taille,&Rotation, Name)<12) return 0;//printf("Chargement de M%u %dh%fmin %ddeg%dmin Vmag=%f Taille=%f Rot:%f\n",Messier,rahr,ramin,dedeg,demin,Mag,Taille,Rotation);

	// Replace the "_" with " "
	char * cc;
	cc=strchr(Name,'_');
	while(cc!=NULL)
	{	(*cc)=' ';
		cc=strchr(Name,'_');
	}
	
	// Calc the RA and DE from the datas
	RaRad=RA_en_Rad(rahr,ramin);
	DecRad=DE_en_Rad(dedeg,demin);

	// Calc the Cartesian coord with RA and DE
	RADE_to_XYZ(RaRad,DecRad,XYZ);
	XYZ*=RAYON;

	matTransfo=new float[16];	// Used to store the precalc transfos matrix used to draw the star

	glLoadIdentity();			// Init the current matrix

	// Precalcul de la matrice de Positionnement/Rotation pour le dessin
	glTranslatef(XYZ[0],XYZ[1],XYZ[2]);
	glRotatef(RaRad*180/PI,0,1,0);
	glRotatef(DecRad*180/PI,-1,0,0);
	glRotatef(Rotation,0,0,1);
	glGetFloatv(GL_MODELVIEW_MATRIX , matTransfo);	// Store the matrix

	RayonPrecalc=RAYON*sin(Taille/2/60*PI/180);
	//printf("rayon : %f\n",RayonPrecalc);

	if (Messier>0)		//Load texture for Messiers
	{	int temp = ReadTexture();
		return temp;
	}
	return 0;
}


int Nebula::ReadTexture()
{	char nomFic[128];
	sprintf(nomFic,"m%d",Messier);
	TexID=createTexture(nomFic);
	return 1;
}

void Nebula::Draw()
{	// Check if in the field of view, if not return
	if (acos((global.XYZVision[0]*XYZ[0]+global.XYZVision[1]*XYZ[1]+global.XYZVision[2]*XYZ[2])/RAYON)>precalculatedFOVNeb) return;

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glPushMatrix();
	glMultMatrixf(matTransfo);			// recuperation de la matrice calcul au chargement

	float cmag=(1-Mag/12);
	glColor3f(cmag,cmag,cmag);
	glBindTexture (GL_TEXTURE_2D, TexID);
	
	glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2i(1,0);				//Bas Droite
		glVertex3f(RayonPrecalc,-RayonPrecalc,0.0f);
		glTexCoord2i(0,0);				//Bas Gauche
		glVertex3f(-RayonPrecalc,-RayonPrecalc,0.0f);
		glTexCoord2i(1,1);				//Haut Droit
		glVertex3f(RayonPrecalc,RayonPrecalc,0.0f);
		glTexCoord2i(0,1);				//Haut Gauche
		glVertex3f(-RayonPrecalc,RayonPrecalc,0.0f);
	glEnd ();

	
	if (global.Fov>sqrt(Taille)*2 && global.FlagNebula && !global.FlagRealMode)
	{	incLum++;
		glColor3f(sqrt(global.Fov)/10*(0.4+0.2*sin(incLum/10)),sqrt(global.Fov)/10*(0.4+0.2*sin(incLum/10)),0.1);
		glBindTexture (GL_TEXTURE_2D, Nebula::texCircleId);	
	
		float r=0.05*global.Fov;
		glBegin(GL_TRIANGLE_STRIP);
			glTexCoord2i(1,0);				//Bas Droite
			glVertex3f(r,-r,0.0f);
			glTexCoord2i(0,0);				//Bas Gauche
			glVertex3f(-r,-r,0.0f);
			glTexCoord2i(1,1);				//Haut Droit
			glVertex3f(r,r,0.0f);
			glTexCoord2i(0,1);				//Haut Gauche
			glVertex3f(-r,r,0.0f);
		glEnd ();
	}
	glPopMatrix();	
}

unsigned int Nebula::texCircleId=0;
